@@ -1,17 +1,17 @@
-use crate::index::{
-    config::{ByteVec, IndexOrd, IndexType, Indexes, ValueMode, WrapperType},
-    serialization::{deserialize, serialize},
-    tree::{
-        nodes::{compare, Node, NodeRef, PageIter, PageIterBack, Value},
-        IndexApply, KeyChanges, ValueChange as TreeValue,
-    },
-};
 use crate::{
-    error::{PRes, PersyError},
+    error::{GenericError, IndexChangeError, PERes, PIRes, ReadError},
     id::{index_id_to_segment_id_data, index_id_to_segment_id_meta, IndexId, SegmentId},
+    index::{
+        config::{ByteVec, IndexOrd, IndexType, Indexes, ValueMode, WrapperType},
+        serialization::{deserialize, serialize},
+        tree::{
+            nodes::{compare, Node, NodeRef, PageIter, PageIterBack, Value},
+            IndexApply, KeyChanges, ValueChange as TreeValue,
+        },
+    },
     persy::PersyImpl,
-    snapshot::SnapshotId,
-    transaction::Transaction,
+    snapshots::SnapshotId,
+    transaction_impl::TransactionImpl,
     PersyId,
 };
 use std::{
@@ -61,14 +61,14 @@ impl<T: IndexOrd> From<T> for Container<T> {
 
 #[derive(Clone, Debug, PartialEq)]
 pub enum ValueChange<V> {
-    ADD(V),
-    REMOVE(Option<V>),
+    Add(V),
+    Remove(Option<V>),
 }
 
 #[derive(Clone)]
 pub enum Change {
-    ADD(usize),
-    REMOVE(Option<usize>),
+    Add(usize),
+    Remove(Option<usize>),
 }
 
 #[derive(Clone)]
@@ -97,8 +97,8 @@ fn resolve_values<V: IndexType>(values: &ValueContainer, changes: Changes) -> Ve
         .changes
         .iter()
         .map(|c| match c {
-            Change::ADD(p) => ValueChange::ADD(v[*p].clone().value()),
-            Change::REMOVE(o) => ValueChange::REMOVE(o.map(|p| v[p].clone().value())),
+            Change::Add(p) => ValueChange::Add(v[*p].clone().value()),
+            Change::Remove(o) => ValueChange::Remove(o.map(|p| v[p].clone().value())),
         })
         .collect()
 }
@@ -200,8 +200,8 @@ macro_rules! container_enums {
             values: &ValueContainer,
             index_id: &IndexId,
             persy: &PersyImpl,
-            tx: &mut Transaction,
-        ) -> PRes<()> {
+            tx: &mut TransactionImpl,
+        ) -> PIRes<()> {
             match keys {
                 $(
                 EntriesContainer::$variant(k) => valapplier::<$t>(values, k, index_id, persy, tx),
@@ -214,8 +214,8 @@ macro_rules! container_enums {
             k: &BTreeMap<K::Wrapper, Changes>,
             index_id: &IndexId,
             persy: &PersyImpl,
-            tx: &mut Transaction,
-        ) -> PRes<()>
+            tx: &mut TransactionImpl,
+        ) -> PIRes<()>
         where
             K: IndexType,
         {
@@ -245,9 +245,9 @@ container_enums!(
     I128<i128>,
     F32W<f32>,
     F64W<f64>,
-    STRING<String>,
-    PERSYID<PersyId>,
-    BYTEVEC<ByteVec>,
+    String<String>,
+    PersyId<PersyId>,
+    ByteVec<ByteVec>,
 );
 
 pub trait Extractor: Sized {
@@ -261,11 +261,11 @@ pub trait Extractor: Sized {
 
 fn apply_to_index<K, V>(
     persy: &PersyImpl,
-    tx: &mut Transaction,
+    tx: &mut TransactionImpl,
     index_id: &IndexId,
     keys: &BTreeMap<K::Wrapper, Changes>,
     values: &[V::Wrapper],
-) -> PRes<()>
+) -> PIRes<()>
 where
     K: IndexType,
     V: IndexType,
@@ -277,8 +277,8 @@ where
                 .changes
                 .iter()
                 .map(|ch| match *ch {
-                    Change::ADD(pos) => TreeValue::ADD(values[pos].clone().value()),
-                    Change::REMOVE(pos) => TreeValue::REMOVE(pos.map(|p| values[p].clone().value())),
+                    Change::Add(pos) => TreeValue::Add(values[pos].clone().value()),
+                    Change::Remove(pos) => TreeValue::Remove(pos.map(|p| values[p].clone().value())),
                 })
                 .collect();
             KeyChanges::new(k.clone().value(), vals)
@@ -299,13 +299,13 @@ fn map_bound<T: Into<T1> + Clone, T1>(b: Bound<&T>) -> Bound<T1> {
 }
 
 pub struct IndexTransactionKeeper {
-    indexex_changes: HashMap<IndexId, (EntriesContainer, ValueContainer)>,
+    indexex_changes: BTreeMap<IndexId, (EntriesContainer, ValueContainer)>,
 }
 
 impl IndexTransactionKeeper {
     pub fn new() -> IndexTransactionKeeper {
         IndexTransactionKeeper {
-            indexex_changes: HashMap::new(),
+            indexex_changes: BTreeMap::new(),
         }
     }
 
@@ -315,16 +315,16 @@ impl IndexTransactionKeeper {
         V: IndexType,
     {
         match self.indexex_changes.entry(index) {
-            Entry::Occupied(ref mut o) => {
+            BTreeEntry::Occupied(ref mut o) => {
                 let (entries, values) = o.get_mut();
                 let pos = add_value(values, v);
-                add_entry(entries, k, Change::ADD(pos));
+                add_entry(entries, k, Change::Add(pos));
             }
-            Entry::Vacant(va) => {
+            BTreeEntry::Vacant(va) => {
                 let mut values = V::Wrapper::new_values();
                 let mut keys = K::Wrapper::new_entries();
                 let pos = add_value(&mut values, v);
-                add_entry(&mut keys, k, Change::ADD(pos));
+                add_entry(&mut keys, k, Change::Add(pos));
                 va.insert((keys, values));
             }
         }
@@ -336,23 +336,15 @@ impl IndexTransactionKeeper {
         V: IndexType,
     {
         match self.indexex_changes.entry(index) {
-            Entry::Occupied(ref mut o) => {
-                let pos = if let Some(val) = v {
-                    Some(add_value(&mut o.get_mut().1, val))
-                } else {
-                    None
-                };
-                add_entry(&mut o.get_mut().0, k, Change::REMOVE(pos));
+            BTreeEntry::Occupied(ref mut o) => {
+                let pos = v.map(|val| add_value(&mut o.get_mut().1, val));
+                add_entry(&mut o.get_mut().0, k, Change::Remove(pos));
             }
-            Entry::Vacant(va) => {
+            BTreeEntry::Vacant(va) => {
                 let mut values = V::Wrapper::new_values();
                 let mut keys = K::Wrapper::new_entries();
-                let pos = if let Some(val) = v {
-                    Some(add_value(&mut values, val))
-                } else {
-                    None
-                };
-                add_entry(&mut keys, k, Change::REMOVE(pos));
+                let pos = v.map(|val| add_value(&mut values, val));
+                add_entry(&mut keys, k, Change::Remove(pos));
                 va.insert((keys, values));
             }
         }
@@ -365,7 +357,7 @@ impl IndexTransactionKeeper {
     {
         self.indexex_changes
             .get(&index)
-            .map(|ref o| get_changes(&o.0, k).map(|c| resolve_values(&o.1, c)))
+            .map(|o| get_changes(&o.0, k).map(|c| resolve_values(&o.1, c)))
             .and_then(std::convert::identity)
     }
 
@@ -375,7 +367,7 @@ impl IndexTransactionKeeper {
         vm: ValueMode,
         k: &K,
         pers: Option<Value<V>>,
-    ) -> PRes<Option<Value<V>>>
+    ) -> Result<Option<Value<V>>, IndexChangeError>
     where
         K: IndexType,
         V: IndexType,
@@ -384,51 +376,54 @@ impl IndexTransactionKeeper {
         if let Some(key_changes) = self.get_changes::<K, V>(index_id, k) {
             for change in key_changes {
                 result = match change {
-                    ValueChange::ADD(add_value) => Some(if let Some(s_result) = result {
+                    ValueChange::Add(add_value) => Some(if let Some(s_result) = result {
                         match s_result {
-                            Value::SINGLE(v) => match vm {
-                                ValueMode::REPLACE => Value::SINGLE(add_value),
-                                ValueMode::EXCLUSIVE => {
+                            Value::Single(v) => match vm {
+                                ValueMode::Replace => Value::Single(add_value),
+                                ValueMode::Exclusive => {
                                     if compare(&v, &add_value) == Ordering::Equal {
-                                        Value::SINGLE(v)
+                                        Value::Single(v)
                                     } else {
                                         // TODO: recover index name
-                                        return Err(PersyError::IndexDuplicateKey("".to_string(), format!("{}", k)));
+                                        return Err(IndexChangeError::IndexDuplicateKey(
+                                            "".to_string(),
+                                            format!("{}", k),
+                                        ));
                                     }
                                 }
-                                ValueMode::CLUSTER => match compare(&v, &add_value) {
-                                    Ordering::Equal => Value::SINGLE(v),
-                                    Ordering::Less => Value::CLUSTER(vec![v, add_value]),
-                                    Ordering::Greater => Value::CLUSTER(vec![add_value, v]),
+                                ValueMode::Cluster => match compare(&v, &add_value) {
+                                    Ordering::Equal => Value::Single(v),
+                                    Ordering::Less => Value::Cluster(vec![v, add_value]),
+                                    Ordering::Greater => Value::Cluster(vec![add_value, v]),
                                 },
                             },
-                            Value::CLUSTER(mut values) => {
+                            Value::Cluster(mut values) => {
                                 if let Err(pos) = values.binary_search_by(|x| compare(x, &add_value)) {
                                     values.insert(pos, add_value);
                                 }
-                                Value::CLUSTER(values)
+                                Value::Cluster(values)
                             }
                         }
                     } else {
-                        Value::SINGLE(add_value)
+                        Value::Single(add_value)
                     }),
-                    ValueChange::REMOVE(rv) => rv.and_then(|remove_value| {
+                    ValueChange::Remove(rv) => rv.and_then(|remove_value| {
                         result.and_then(|s_result| match s_result {
-                            Value::SINGLE(v) => {
+                            Value::Single(v) => {
                                 if compare(&v, &remove_value) == Ordering::Equal {
                                     None
                                 } else {
-                                    Some(Value::SINGLE(v))
+                                    Some(Value::Single(v))
                                 }
                             }
-                            Value::CLUSTER(mut values) => {
+                            Value::Cluster(mut values) => {
                                 if let Ok(pos) = values.binary_search_by(|x| compare(x, &remove_value)) {
                                     values.remove(pos);
                                 }
                                 Some(if values.len() == 1 {
-                                    Value::SINGLE(values.pop().unwrap())
+                                    Value::Single(values.pop().unwrap())
                                 } else {
-                                    Value::CLUSTER(values)
+                                    Value::Cluster(values)
                                 })
                             }
                         })
@@ -439,13 +434,9 @@ impl IndexTransactionKeeper {
         Ok(result)
     }
 
-    pub fn apply(&self, persy: &PersyImpl, tx: &mut Transaction) -> PRes<()> {
-        let mut indexes = self.changed_indexes();
-        indexes.sort();
-        for index in indexes {
-            if let Some(values) = self.indexex_changes.get(&index) {
-                eapplier(&values.0, &values.1, &index, persy, tx)?;
-            }
+    pub fn apply(&self, persy: &PersyImpl, tx: &mut TransactionImpl) -> PIRes<()> {
+        for (index, (keys, values)) in &self.indexex_changes {
+            eapplier(keys, values, index, persy, tx)?;
         }
         Ok(())
     }
@@ -468,24 +459,24 @@ impl IndexTransactionKeeper {
 }
 
 pub trait IndexKeeper<K, V> {
-    fn load(&self, node: &NodeRef) -> PRes<Node<K, V>>;
-    fn get_root(&self) -> PRes<Option<NodeRef>>;
+    fn load(&self, node: &NodeRef) -> PERes<Node<K, V>>;
+    fn get_root(&self) -> PERes<Option<NodeRef>>;
     fn value_mode(&self) -> ValueMode;
     fn index_name(&self) -> &String;
 }
 
 pub trait IndexModify<K, V>: IndexKeeper<K, V> {
-    fn load_modify(&self, node: &NodeRef) -> PRes<Option<(Rc<Node<K, V>>, u16)>>;
-    fn lock(&mut self, node: &NodeRef, version: u16) -> PRes<bool>;
+    fn load_modify(&self, node: &NodeRef) -> PIRes<Option<(Rc<Node<K, V>>, u16)>>;
+    fn lock(&mut self, node: &NodeRef, version: u16) -> PIRes<bool>;
     fn owned(&mut self, node_ref: &NodeRef, node: Rc<Node<K, V>>) -> Node<K, V>;
-    fn unlock(&mut self, node: &NodeRef) -> PRes<bool>;
-    fn unlock_config(&mut self) -> PRes<bool>;
-    fn get_root_refresh(&mut self) -> PRes<Option<NodeRef>>;
-    fn lock_config(&mut self) -> PRes<bool>;
-    fn insert(&mut self, node: Node<K, V>) -> PRes<NodeRef>;
-    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, version: u16) -> PRes<()>;
-    fn delete(&mut self, node: &NodeRef, version: u16) -> PRes<()>;
-    fn set_root(&mut self, root: Option<NodeRef>) -> PRes<()>;
+    fn unlock(&mut self, node: &NodeRef) -> PIRes<bool>;
+    fn unlock_config(&mut self) -> PIRes<bool>;
+    fn get_root_refresh(&mut self) -> PIRes<Option<NodeRef>>;
+    fn lock_config(&mut self) -> PIRes<bool>;
+    fn insert(&mut self, node: Node<K, V>) -> PIRes<NodeRef>;
+    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, version: u16) -> PIRes<()>;
+    fn delete(&mut self, node: &NodeRef, version: u16) -> PIRes<()>;
+    fn set_root(&mut self, root: Option<NodeRef>) -> PIRes<()>;
     fn bottom_limit(&self) -> usize;
     fn top_limit(&self) -> usize;
 }
@@ -519,15 +510,24 @@ impl<'a> IndexSegmentKeeper<'a> {
     }
 }
 
+fn map_read_err(r: ReadError) -> GenericError {
+    match r {
+        ReadError::SegmentNotFound => panic!("The segment should be already checked"),
+        ReadError::InvalidPersyId(_) => panic!("The Internal id should be everytime valid"),
+        ReadError::Generic(g) => g,
+    }
+}
+
 impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeper<'a> {
-    fn load(&self, node: &NodeRef) -> PRes<Node<K, V>> {
+    fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         let rec = self
             .store
-            .read_snap_fn(self.segment, &node, self.snapshot, |x| deserialize(x))?
+            .read_snap_fn(self.segment, node, self.snapshot, |x| deserialize(x))
+            .map_err(map_read_err)?
             .unwrap()?;
         Ok(rec)
     }
-    fn get_root(&self) -> PRes<Option<NodeRef>> {
+    fn get_root(&self) -> PERes<Option<NodeRef>> {
         Ok(self.root.clone())
     }
     fn value_mode(&self) -> ValueMode {
@@ -550,7 +550,7 @@ pub struct IndexSegmentKeeperTx<'a, K: IndexType, V: IndexType> {
     root: Option<NodeRef>,
     config_version: u16,
     store: &'a PersyImpl,
-    tx: &'a mut Transaction,
+    tx: &'a mut TransactionImpl,
     value_mode: ValueMode,
     changed: Option<HashMap<NodeRef, (Rc<Node<K, V>>, u16)>>,
     bottom_limit: usize,
@@ -566,7 +566,7 @@ impl<'a, K: IndexType, V: IndexType> IndexSegmentKeeperTx<'a, K, V> {
         root: Option<NodeRef>,
         config_version: u16,
         store: &'a PersyImpl,
-        tx: &'a mut Transaction,
+        tx: &'a mut TransactionImpl,
         value_mode: ValueMode,
         bottom_limit: usize,
         top_limit: usize,
@@ -586,11 +586,11 @@ impl<'a, K: IndexType, V: IndexType> IndexSegmentKeeperTx<'a, K, V> {
             updated_root: false,
         }
     }
-    pub fn update_changed(&mut self) -> PRes<()> {
+    pub fn update_changed(&mut self) -> PIRes<()> {
         let segment = index_id_to_segment_id_data(&self.index_id);
         if let Some(m) = &self.changed {
             for (node_ref, node) in m {
-                self.store.update(self.tx, segment, &node_ref, &serialize(&node.0)?)?;
+                self.store.update(self.tx, segment, node_ref, &serialize(&node.0)?)?;
             }
         }
         if self.updated_root {
@@ -601,7 +601,7 @@ impl<'a, K: IndexType, V: IndexType> IndexSegmentKeeperTx<'a, K, V> {
 }
 
 impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<'a, K, V> {
-    fn load_modify(&self, node: &NodeRef) -> PRes<Option<(Rc<Node<K, V>>, u16)>> {
+    fn load_modify(&self, node: &NodeRef) -> PIRes<Option<(Rc<Node<K, V>>, u16)>> {
         if let Some(m) = &self.changed {
             if let Some(n) = m.get(node) {
                 return Ok(Some(n.clone()));
@@ -610,14 +610,15 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         let segment = index_id_to_segment_id_data(&self.index_id);
         if let Some((rec, version)) = self
             .store
-            .read_tx_internal_fn(self.tx, segment, &node, |x| deserialize(x))?
+            .read_tx_internal_fn(self.tx, segment, node, |x| deserialize(x))
+            .map_err(map_read_err)?
         {
             Ok(Some((Rc::new(rec?), version)))
         } else {
             Ok(None)
         }
     }
-    fn lock(&mut self, node: &NodeRef, version: u16) -> PRes<bool> {
+    fn lock(&mut self, node: &NodeRef, version: u16) -> PIRes<bool> {
         if let Some(lock_data) = self.locked.get_mut(node) {
             if version == lock_data.version {
                 lock_data.counter += 1;
@@ -627,7 +628,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
             }
         } else {
             let segment = index_id_to_segment_id_data(&self.index_id);
-            if self.store.lock_record(self.tx, &segment, node, version)? {
+            if self.store.lock_record(self.tx, segment, node, version)? {
                 self.locked.insert(node.clone(), LockData { version, counter: 1 });
                 Ok(true)
             } else {
@@ -645,13 +646,13 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         Rc::try_unwrap(node).ok().unwrap()
     }
 
-    fn unlock(&mut self, node: &NodeRef) -> PRes<bool> {
+    fn unlock(&mut self, node: &NodeRef) -> PIRes<bool> {
         if let Entry::Occupied(mut x) = self.locked.entry(node.clone()) {
             x.get_mut().counter -= 1;
             if x.get().counter == 0 {
                 x.remove();
                 let segment = index_id_to_segment_id_data(&self.index_id);
-                self.store.unlock_record(self.tx, &segment, node)?;
+                self.store.unlock_record(self.tx, segment, node)?;
                 Ok(true)
             } else {
                 Ok(false)
@@ -661,7 +662,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         }
     }
 
-    fn get_root_refresh(&mut self) -> PRes<Option<NodeRef>> {
+    fn get_root_refresh(&mut self) -> PIRes<Option<NodeRef>> {
         if !self.updated_root {
             let (config, version) = Indexes::get_index_tx(self.store, self.tx, &self.index_id)?;
             self.root = config.get_root();
@@ -669,14 +670,14 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         }
         Ok(self.root.clone())
     }
-    fn unlock_config(&mut self) -> PRes<bool> {
+    fn unlock_config(&mut self) -> PIRes<bool> {
         let config_id = Indexes::get_config_id(self.store, self.tx, &self.index_id)?.0;
         if let Entry::Occupied(mut x) = self.locked.entry(config_id.clone()) {
             x.get_mut().counter -= 1;
             if x.get().counter == 0 {
                 x.remove();
                 let segment = index_id_to_segment_id_meta(&self.index_id);
-                self.store.unlock_record(self.tx, &segment, &config_id)?;
+                self.store.unlock_record(self.tx, segment, &config_id)?;
                 Ok(true)
             } else {
                 Ok(false)
@@ -685,7 +686,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
             Ok(false)
         }
     }
-    fn lock_config(&mut self) -> PRes<bool> {
+    fn lock_config(&mut self) -> PIRes<bool> {
         let config_id = Indexes::get_config_id(self.store, self.tx, &self.index_id)?.0;
 
         let segment = index_id_to_segment_id_meta(&self.index_id);
@@ -698,7 +699,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
             }
         } else if self
             .store
-            .lock_record(self.tx, &segment, &config_id, self.config_version)?
+            .lock_record(self.tx, segment, &config_id, self.config_version)?
         {
             self.locked.insert(
                 config_id.clone(),
@@ -716,7 +717,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         }
     }
 
-    fn insert(&mut self, node: Node<K, V>) -> PRes<NodeRef> {
+    fn insert(&mut self, node: Node<K, V>) -> PIRes<NodeRef> {
         let segment = index_id_to_segment_id_data(&self.index_id);
         let node_ref = self.store.insert_record(self.tx, &segment, &serialize(&node)?)?;
         self.changed
@@ -727,7 +728,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         Ok(node_ref)
     }
 
-    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, version: u16) -> PRes<()> {
+    fn update(&mut self, node_ref: &NodeRef, node: Node<K, V>, version: u16) -> PIRes<()> {
         debug_assert!(self.locked.contains_key(node_ref));
         self.changed
             .get_or_insert_with(HashMap::new)
@@ -735,16 +736,16 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
         Ok(())
     }
 
-    fn delete(&mut self, node: &NodeRef, _version: u16) -> PRes<()> {
+    fn delete(&mut self, node: &NodeRef, _version: u16) -> PIRes<()> {
         debug_assert!(self.locked.contains_key(node));
         if let Some(m) = &mut self.changed {
             m.remove(node);
         }
         let segment = index_id_to_segment_id_data(&self.index_id);
-        self.store.delete(self.tx, segment, &node)?;
+        self.store.delete(self.tx, segment, node)?;
         Ok(())
     }
-    fn set_root(&mut self, root: Option<NodeRef>) -> PRes<()> {
+    fn set_root(&mut self, root: Option<NodeRef>) -> PIRes<()> {
         self.root = root;
         self.updated_root = true;
         Ok(())
@@ -759,7 +760,7 @@ impl<'a, K: IndexType, V: IndexType> IndexModify<K, V> for IndexSegmentKeeperTx<
 }
 
 impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeperTx<'a, K, V> {
-    fn load(&self, node: &NodeRef) -> PRes<Node<K, V>> {
+    fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         if let Some(m) = &self.changed {
             if let Some(n) = m.get(node) {
                 return Ok(n.0.as_ref().clone());
@@ -768,11 +769,12 @@ impl<'a, K: IndexType, V: IndexType> IndexKeeper<K, V> for IndexSegmentKeeperTx<
         let segment = index_id_to_segment_id_data(&self.index_id);
         let (rec, _) = self
             .store
-            .read_tx_internal_fn(self.tx, segment, &node, |x| deserialize(x))?
+            .read_tx_internal_fn(self.tx, segment, node, |x| deserialize(x))
+            .map_err(map_read_err)?
             .unwrap();
-        Ok(rec?)
+        rec
     }
-    fn get_root(&self) -> PRes<Option<NodeRef>> {
+    fn get_root(&self) -> PERes<Option<NodeRef>> {
         Ok(self.root.clone())
     }
     fn value_mode(&self) -> ValueMode {
@@ -821,7 +823,7 @@ where
     }
 
     fn apply_changes(
-        tx: &mut Transaction,
+        tx: &mut TransactionImpl,
         vm: ValueMode,
         index: IndexId,
         k: K,
@@ -830,7 +832,7 @@ where
         tx.apply_changes(vm, index, &k, pers).unwrap_or(None).map(|v| (k, v))
     }
 
-    pub fn next(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut Transaction) -> Option<(K, Value<V>)> {
+    pub fn next(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut TransactionImpl) -> Option<(K, Value<V>)> {
         loop {
             let vm = self.value_mode.clone();
             let index = self.index_name.clone();
@@ -891,7 +893,7 @@ where
         }
     }
 
-    pub fn next_back(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut Transaction) -> Option<(K, Value<V>)> {
+    pub fn next_back(&mut self, persy_impl: &Arc<PersyImpl>, tx: &mut TransactionImpl) -> Option<(K, Value<V>)> {
         loop {
             let vm = self.value_mode.clone();
             let index = self.index_name.clone();
@@ -1020,7 +1022,7 @@ where
             None
         }
     }
-    pub fn release(&self, persy_impl: &PersyImpl) -> PRes<()> {
+    pub fn release(&self, persy_impl: &PersyImpl) -> PERes<()> {
         if self.release_snapshot {
             persy_impl.release_snapshot(self.read_snapshot)?;
         }
@@ -1039,10 +1041,10 @@ mod tests {
         let mut keeper = IndexTransactionKeeper::new();
         keeper.put(name.clone(), k.clone(), v.clone());
         let ret = keeper.get_changes(name.clone(), &k);
-        assert_eq!(ret, Some(vec![ValueChange::ADD(v.clone())]));
+        assert_eq!(ret, Some(vec![ValueChange::Add(v.clone())]));
         keeper.remove(name.clone(), dk.clone(), Some(v.clone()));
         let ret = keeper.get_changes(name, &dk);
-        assert_eq!(ret, Some(vec![ValueChange::REMOVE(Some(v))]));
+        assert_eq!(ret, Some(vec![ValueChange::Remove(Some(v))]));
     }
 
     #[test]
