@@ -1,19 +1,19 @@
-use crate::transaction::{
+use crate::transaction_impl::{
     Cleanup, Commit, CreateSegment, DeleteRecord, DropSegment, FreedPage, InsertRecord, Metadata, NewSegmentPage,
-    PrepareCommit, ReadRecord, Rollback, Transaction, UpdateRecord,
+    PrepareCommit, ReadRecord, Rollback, TransactionImpl, UpdateRecord,
 };
 use crate::{
     allocator::Allocator,
     config::TxStrategy,
     discref::{Page, PageOps, ReadPage, PAGE_METADATA_SIZE},
-    error::PRes,
+    error::PERes,
     flush_checksum::double_buffer_check,
     id::SegmentId,
     io::{
         read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleReadVarInt, InfallibleWrite,
         InfallibleWriteFormat, InfallibleWriteVarInt,
     },
-    persy::RecoverStatus,
+    RecoverStatus,
 };
 use std::{
     collections::hash_map::HashMap,
@@ -107,9 +107,9 @@ pub struct Journal {
 
 pub(crate) trait JournalEntry {
     fn get_type(&self) -> u8;
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()>;
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()>;
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus>;
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()>;
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()>;
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus>;
 }
 
 pub struct Start {}
@@ -120,7 +120,7 @@ pub struct JournalId {
     pos: u32,
 }
 
-fn recover_entry<T>(entry: &mut dyn JournalEntry, page: &mut ReadPage, found: &mut T, id: &JournalId) -> PRes<()>
+fn recover_entry<T>(entry: &mut dyn JournalEntry, page: &mut ReadPage, found: &mut T, id: &JournalId) -> PERes<()>
 where
     T: FnMut(&dyn JournalEntry, &JournalId),
 {
@@ -130,7 +130,7 @@ where
 }
 
 impl Journal {
-    pub fn new(all: &Arc<Allocator>, page: u64) -> PRes<Journal> {
+    pub fn new(all: &Arc<Allocator>, page: u64) -> PERes<Journal> {
         let mut page = all.load_page(page)?;
         let journal = match page.read_u8() {
             JOURNAL_ROOT_VERSION_0 => Self::new_version_0(page, all)?,
@@ -141,7 +141,7 @@ impl Journal {
             journal: Mutex::new(journal),
         })
     }
-    fn new_version_0(mut page: ReadPage, all: &Allocator) -> PRes<JournalShared> {
+    fn new_version_0(mut page: ReadPage, all: &Allocator) -> PERes<JournalShared> {
         let first_page;
         let last_flush;
         let mut buffer_0 = [0; 11];
@@ -174,7 +174,7 @@ impl Journal {
         })
     }
 
-    pub fn init(allocator: &Allocator) -> PRes<u64> {
+    pub fn init(allocator: &Allocator) -> PERes<u64> {
         let root_page = allocator.allocate(5)?;
         let root_page_index = root_page.get_index();
         let first_page = allocator.allocate(JOURNAL_PAGE_EXP)?;
@@ -184,25 +184,25 @@ impl Journal {
         Ok(root_page_index)
     }
 
-    pub fn start(&self) -> PRes<JournalId> {
+    pub fn start(&self) -> PERes<JournalId> {
         let val = self.internal_log(&Start::new(), &JournalId::new(0, 0), false)?;
         let id = JournalId::new(val.0, val.1);
         self.journal.lock()?.starts.push(&id);
         Ok(id)
     }
 
-    pub(crate) fn prepare(&self, entry: &dyn JournalEntry, id: &JournalId) -> PRes<()> {
+    pub(crate) fn prepare(&self, entry: &dyn JournalEntry, id: &JournalId) -> PERes<()> {
         self.internal_log(entry, id, true)?;
         Ok(())
     }
 
-    pub(crate) fn end(&self, entry: &dyn JournalEntry, id: &JournalId) -> PRes<()> {
+    pub(crate) fn end(&self, entry: &dyn JournalEntry, id: &JournalId) -> PERes<()> {
         self.internal_log(entry, id, true)?;
         // TODO:optimization keep in memory last page and move the flush here.
         Ok(())
     }
 
-    pub fn clear_in_queque(&self) -> PRes<()> {
+    pub fn clear_in_queque(&self) -> PERes<()> {
         let mut pages_to_free = Vec::new();
         {
             let mut lock = self.journal.lock()?;
@@ -244,7 +244,7 @@ impl Journal {
         self.free_pages_tx(pages_to_free)?;
         Ok(())
     }
-    pub fn free_pages_tx(&self, pages_to_free: Vec<u64>) -> PRes<()> {
+    pub fn free_pages_tx(&self, pages_to_free: Vec<u64>) -> PERes<()> {
         let mut prev_tx_id = None;
         {
             //Free the pages of previous clear, logged 2 tx ahead, now synced because of the second
@@ -279,7 +279,7 @@ impl Journal {
         Ok(())
     }
 
-    pub fn finished_to_clean(&self, ids: &[JournalId]) -> PRes<()> {
+    pub fn finished_to_clean(&self, ids: &[JournalId]) -> PERes<()> {
         for id in ids {
             self.log(&Cleanup::new(), id)?;
         }
@@ -288,18 +288,18 @@ impl Journal {
         Ok(())
     }
 
-    pub fn cleaned_to_trim(&self, ids: &[JournalId]) -> PRes<()> {
+    pub fn cleaned_to_trim(&self, ids: &[JournalId]) -> PERes<()> {
         let mut lock = self.journal.lock()?;
         lock.to_clear.extend_from_slice(ids);
         Ok(())
     }
 
-    pub(crate) fn log(&self, entry: &dyn JournalEntry, id: &JournalId) -> PRes<()> {
+    pub(crate) fn log(&self, entry: &dyn JournalEntry, id: &JournalId) -> PERes<()> {
         self.internal_log(entry, id, false)?;
         Ok(())
     }
 
-    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PRes<(u64, u32)> {
+    fn internal_log(&self, entry: &dyn JournalEntry, id: &JournalId, flush: bool) -> PERes<(u64, u32)> {
         let mut buffer = Vec::<u8>::new();
         buffer.write_u8(entry.get_type());
         buffer.write_varint_u64(id.page);
@@ -323,7 +323,7 @@ impl Journal {
         Ok((cur_page, cur_pos))
     }
 
-    pub(crate) fn recover<T>(&self, mut found: T) -> PRes<()>
+    pub(crate) fn recover<T>(&self, mut found: T) -> PERes<()>
     where
         T: FnMut(&dyn JournalEntry, &JournalId),
     {
@@ -380,7 +380,7 @@ impl Journal {
         Ok(())
     }
 
-    fn required_space(&self, space: u32, jr: &mut MutexGuard<JournalShared>) -> PRes<()> {
+    fn required_space(&self, space: u32, jr: &mut MutexGuard<JournalShared>) -> PERes<()> {
         // if there is no page or the  'current content' + 'space required' + 'end marker' is more
         // than the page, allocate new page and link the previous one
         if jr.last_pos + space + 1 >= JOURNAL_PAGE_SIZE as u32 {
@@ -410,7 +410,7 @@ impl JournalEntry for DeleteRecord {
         6
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment.write_varint(buffer);
         buffer.write_varint_u64(self.recref.page);
         buffer.write_varint_u32(self.recref.pos);
@@ -418,7 +418,7 @@ impl JournalEntry for DeleteRecord {
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment = SegmentId::read_varint(buffer);
         self.recref.page = buffer.read_varint_u64();
         self.recref.pos = buffer.read_varint_u32();
@@ -426,7 +426,7 @@ impl JournalEntry for DeleteRecord {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_delete(self);
         Ok(RecoverStatus::Started)
     }
@@ -437,7 +437,7 @@ impl JournalEntry for ReadRecord {
         10
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment.write_varint(buffer);
         buffer.write_varint_u64(self.recref.page);
         buffer.write_varint_u32(self.recref.pos);
@@ -445,7 +445,7 @@ impl JournalEntry for ReadRecord {
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment = SegmentId::read_varint(buffer);
         self.recref.page = buffer.read_varint_u64();
         self.recref.pos = buffer.read_varint_u32();
@@ -453,7 +453,7 @@ impl JournalEntry for ReadRecord {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_read(self);
         Ok(RecoverStatus::Started)
     }
@@ -463,7 +463,7 @@ impl JournalEntry for UpdateRecord {
         5
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment.write_varint(buffer);
         buffer.write_varint_u64(self.recref.page);
         buffer.write_varint_u32(self.recref.pos);
@@ -472,7 +472,7 @@ impl JournalEntry for UpdateRecord {
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment = SegmentId::read_varint(buffer);
         self.recref.page = buffer.read_varint_u64();
         self.recref.pos = buffer.read_varint_u32();
@@ -481,7 +481,7 @@ impl JournalEntry for UpdateRecord {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_update(self);
         Ok(RecoverStatus::Started)
     }
@@ -492,7 +492,7 @@ impl JournalEntry for InsertRecord {
         2
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment.write_varint(buffer);
         buffer.write_varint_u64(self.recref.page);
         buffer.write_varint_u32(self.recref.pos);
@@ -500,7 +500,7 @@ impl JournalEntry for InsertRecord {
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment = SegmentId::read_varint(buffer);
         self.recref.page = buffer.read_varint_u64();
         self.recref.pos = buffer.read_varint_u32();
@@ -508,7 +508,7 @@ impl JournalEntry for InsertRecord {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_insert(self);
         Ok(RecoverStatus::Started)
     }
@@ -519,15 +519,15 @@ impl JournalEntry for PrepareCommit {
         3
     }
 
-    fn write(&self, _: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, _: &mut dyn InfallibleWrite) -> PERes<()> {
         Ok(())
     }
 
-    fn read(&mut self, _: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, _: &mut dyn InfallibleRead) -> PERes<()> {
         Ok(())
     }
 
-    fn recover(&self, _: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, _: &mut TransactionImpl) -> PERes<RecoverStatus> {
         Ok(RecoverStatus::PrepareCommit)
     }
 }
@@ -537,15 +537,15 @@ impl JournalEntry for Commit {
         4
     }
 
-    fn write(&self, _: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, _: &mut dyn InfallibleWrite) -> PERes<()> {
         Ok(())
     }
 
-    fn read(&mut self, _: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, _: &mut dyn InfallibleRead) -> PERes<()> {
         Ok(())
     }
 
-    fn recover(&self, _: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, _: &mut TransactionImpl) -> PERes<RecoverStatus> {
         Ok(RecoverStatus::Commit)
     }
 }
@@ -555,15 +555,15 @@ impl JournalEntry for Cleanup {
         14
     }
 
-    fn write(&self, _: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, _: &mut dyn InfallibleWrite) -> PERes<()> {
         Ok(())
     }
 
-    fn read(&mut self, _: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, _: &mut dyn InfallibleRead) -> PERes<()> {
         Ok(())
     }
 
-    fn recover(&self, _: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, _: &mut TransactionImpl) -> PERes<RecoverStatus> {
         Ok(RecoverStatus::Cleanup)
     }
 }
@@ -573,7 +573,7 @@ impl JournalEntry for Metadata {
         11
     }
 
-    fn write(&self, write: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, write: &mut dyn InfallibleWrite) -> PERes<()> {
         write.write_varint_u8(self.strategy.value());
         let len = self.meta_id.len();
         write.write_varint_u16(len as u16);
@@ -581,7 +581,7 @@ impl JournalEntry for Metadata {
         Ok(())
     }
 
-    fn read(&mut self, read: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, read: &mut dyn InfallibleRead) -> PERes<()> {
         self.strategy = TxStrategy::from_value(read.read_varint_u8());
         let len = read.read_varint_u16();
         let mut slice: Vec<u8> = vec![0; len as usize];
@@ -590,7 +590,7 @@ impl JournalEntry for Metadata {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_metadata(self);
         Ok(RecoverStatus::Started)
     }
@@ -601,15 +601,15 @@ impl JournalEntry for Start {
         1
     }
 
-    fn write(&self, _: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, _: &mut dyn InfallibleWrite) -> PERes<()> {
         Ok(())
     }
 
-    fn read(&mut self, _: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, _: &mut dyn InfallibleRead) -> PERes<()> {
         Ok(())
     }
 
-    fn recover(&self, _: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, _: &mut TransactionImpl) -> PERes<RecoverStatus> {
         panic!("this should never be called")
     }
 }
@@ -619,14 +619,14 @@ impl JournalEntry for Rollback {
         7
     }
 
-    fn write(&self, _: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, _: &mut dyn InfallibleWrite) -> PERes<()> {
         Ok(())
     }
 
-    fn read(&mut self, _: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, _: &mut dyn InfallibleRead) -> PERes<()> {
         Ok(())
     }
-    fn recover(&self, _: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, _: &mut TransactionImpl) -> PERes<RecoverStatus> {
         Ok(RecoverStatus::Rollback)
     }
 }
@@ -636,7 +636,7 @@ impl JournalEntry for CreateSegment {
         8
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment_id.write_varint(buffer);
         buffer.write_varint_u64(self.first_page);
         buffer.write_varint_u16(self.name.len() as u16);
@@ -644,7 +644,7 @@ impl JournalEntry for CreateSegment {
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment_id = SegmentId::read_varint(buffer);
         self.first_page = buffer.read_varint_u64();
         let string_size = buffer.read_varint_u16();
@@ -654,7 +654,7 @@ impl JournalEntry for CreateSegment {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_add(self);
         Ok(RecoverStatus::Started)
     }
@@ -665,14 +665,14 @@ impl JournalEntry for DropSegment {
         9
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment_id.write_varint(buffer);
         buffer.write_varint_u16(self.name.len() as u16);
         buffer.write_all(self.name.as_bytes());
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment_id = SegmentId::read_varint(buffer);
         let string_size = buffer.read_varint_u16();
         let mut slice: Vec<u8> = vec![0; string_size as usize];
@@ -681,7 +681,7 @@ impl JournalEntry for DropSegment {
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_drop(self);
         Ok(RecoverStatus::Started)
     }
@@ -692,17 +692,17 @@ impl JournalEntry for FreedPage {
         12
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         buffer.write_varint_u64(self.page);
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.page = buffer.read_varint_u64();
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_freed_page(self);
         Ok(RecoverStatus::Started)
     }
@@ -713,21 +713,21 @@ impl JournalEntry for NewSegmentPage {
         13
     }
 
-    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PRes<()> {
+    fn write(&self, buffer: &mut dyn InfallibleWrite) -> PERes<()> {
         self.segment.write_varint(buffer);
         buffer.write_varint_u64(self.page);
         buffer.write_varint_u64(self.previous);
         Ok(())
     }
 
-    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PRes<()> {
+    fn read(&mut self, buffer: &mut dyn InfallibleRead) -> PERes<()> {
         self.segment = SegmentId::read_varint(buffer);
         self.page = buffer.read_varint_u64();
         self.previous = buffer.read_varint_u64();
         Ok(())
     }
 
-    fn recover(&self, tx: &mut Transaction) -> PRes<RecoverStatus> {
+    fn recover(&self, tx: &mut TransactionImpl) -> PERes<RecoverStatus> {
         tx.recover_new_segment_page(self);
         Ok(RecoverStatus::Started)
     }
@@ -747,7 +747,7 @@ impl Start {
 #[cfg(test)]
 mod tests {
     use super::{Journal, JournalEntry, JournalId, StartList};
-    use crate::transaction::{
+    use crate::transaction_impl::{
         CreateSegment, DeleteRecord, DropSegment, InsertRecord, Metadata, NewSegmentPage, ReadRecord, UpdateRecord,
     };
     use crate::{
