@@ -10,39 +10,40 @@ use std::{
     sync::Arc,
 };
 
-pub use crate::id::RecRef;
-use crate::index::{
-    config::{IndexType, IndexTypeId, Indexes, ValueMode, INDEX_DATA_PREFIX, INDEX_META_PREFIX},
-    keeper::{IndexKeeper, IndexRawIter},
-    tree::{
-        nodes::{PageIter, PageIterBack, Value},
-        Index,
-    },
-};
-use crate::transaction::{
-    PreparedState, SyncMode, Transaction, TxRead,
-    TxSegCheck::{CREATED, DROPPED, NONE},
-};
 use crate::{
     address::Address,
     allocator::Allocator,
     config::{Config, TransactionConfig},
     discref::{Device, DiscRef, MemRef, PageOps, PAGE_METADATA_SIZE},
-    error::{PRes, PersyError},
-    id::{IndexId, PersyId, SegmentId, ToIndexId, ToSegmentId},
+    error::{
+        CreateError, CreateIndexError, CreateSegmentError, DeleteError, DropIndexError, DropSegmentError,
+        IndexChangeError, IndexError, IndexOpsError, OpenError, PERes, SegmentError, TimeoutError, UpdateError,
+    },
+    id::{IndexId, PersyId, RecRef, SegmentId, ToIndexId, ToSegmentId},
+    index::{
+        config::{IndexType, IndexTypeId, Indexes, ValueMode, INDEX_DATA_PREFIX, INDEX_META_PREFIX},
+        keeper::{IndexKeeper, IndexRawIter},
+        tree::{
+            nodes::{PageIter, PageIterBack, Value},
+            Index,
+        },
+    },
     io::{
         InfallibleRead, InfallibleReadFormat, InfallibleReadVarInt, InfallibleWrite, InfallibleWriteFormat,
         InfallibleWriteVarInt,
     },
     journal::{Journal, JournalId},
     record_scanner::{SegmentRawIter, SegmentSnapshotRawIter, TxSegmentRawIter},
-    snapshot::{release_snapshot, EntryCase, SegmentSnapshop, SnapshotId, Snapshots},
+    snapshots::{release_snapshot, EntryCase, SegmentSnapshop, SnapshotId, Snapshots},
+    transaction_impl::{PreparedState, SyncMode, TransactionImpl, TxRead, TxSegCheck},
+    PrepareError, ReadError, RecoverStatus,
 };
 
 #[cfg(feature = "background_ops")]
 use crate::background::BackgroundOps;
 
 const DEFAULT_PAGE_EXP: u8 = 10; // 2^10
+const PERSY_TAG_BYTES: &[u8; 4] = b"prsy"; // 2^10
 
 pub struct PersyImpl {
     config: Arc<Config>,
@@ -56,25 +57,10 @@ pub struct PersyImpl {
 }
 
 pub struct TxFinalize {
-    transaction: Transaction,
+    transaction: TransactionImpl,
     prepared: PreparedState,
 }
 
-/// Possible state of a transaction in the log
-#[derive(PartialEq, Debug, Clone)]
-pub enum RecoverStatus {
-    /// Started but not completed
-    Started,
-    /// Successfully prepared
-    PrepareCommit,
-    /// rollback-ed
-    Rollback,
-    /// Successfully committed after prepared
-    Commit,
-    /// Successfully cleaned up resources after commit
-    Cleanup,
-}
-
 /// Possible option for recover a transaction in prepared commit state
 #[derive(PartialEq, Debug)]
 pub enum CommitStatus {
@@ -83,28 +69,28 @@ pub enum CommitStatus {
 }
 pub struct RecoverImpl {
     tx_id: HashMap<Vec<u8>, JournalId>,
-    transactions: HashMap<JournalId, (RecoverStatus, Transaction, Option<CommitStatus>)>,
+    transactions: HashMap<JournalId, (RecoverStatus, TransactionImpl, Option<CommitStatus>)>,
     order: Vec<JournalId>,
 }
 
 impl RecoverImpl {
-    pub fn apply<C>(&mut self, recover: C) -> PRes<()>
+    pub fn apply<C>(&mut self, recover: C) -> PERes<()>
     where
         C: Fn(&Vec<u8>) -> bool,
     {
-        for (id, status) in self.list_transactions()? {
+        for (id, status) in self.list_transactions() {
             if status == RecoverStatus::PrepareCommit {
                 if recover(&id) {
-                    self.commit(id)?;
+                    self.commit(id);
                 } else {
-                    self.rollback(id)?;
+                    self.rollback(id);
                 }
             }
         }
         Ok(())
     }
 
-    pub fn list_transactions(&self) -> PRes<Vec<(Vec<u8>, RecoverStatus)>> {
+    pub fn list_transactions(&self) -> Vec<(Vec<u8>, RecoverStatus)> {
         let mut res = Vec::new();
         for id in &self.order {
             if let Some((id, status)) = self
@@ -115,33 +101,31 @@ impl RecoverImpl {
                 res.push((id, status));
             }
         }
-        Ok(res)
+        res
     }
 
-    pub fn status(&self, tx_id: Vec<u8>) -> PRes<Option<RecoverStatus>> {
+    pub fn status(&self, tx_id: Vec<u8>) -> Option<RecoverStatus> {
         if let Some(id) = self.tx_id.get(&tx_id) {
-            Ok(self.transactions.get(id).map(|(s, _, _)| s.clone()))
+            self.transactions.get(id).map(|(s, _, _)| s.clone())
         } else {
-            Ok(None)
+            None
         }
     }
 
-    pub fn commit(&mut self, tx_id: Vec<u8>) -> PRes<()> {
+    pub fn commit(&mut self, tx_id: Vec<u8>) {
         if let Some(id) = self.tx_id.get(&tx_id) {
             if let Some(tx) = self.transactions.get_mut(id) {
                 tx.2 = Some(CommitStatus::Commit);
             }
         }
-        Ok(())
     }
 
-    pub fn rollback(&mut self, tx_id: Vec<u8>) -> PRes<()> {
+    pub fn rollback(&mut self, tx_id: Vec<u8>) {
         if let Some(id) = self.tx_id.get(&tx_id) {
             if let Some(tx) = self.transactions.get_mut(id) {
                 tx.2 = Some(CommitStatus::Rollback);
             }
         }
-        Ok(())
     }
 }
 
@@ -159,7 +143,7 @@ fn create_background_ops(
     journal: Arc<Journal>,
     allocator: Arc<Allocator>,
     snapshots: Arc<Snapshots>,
-) -> PRes<BackgroundOps<SnapshotId>> {
+) -> PERes<BackgroundOps<SnapshotId>> {
     let all_sync = allocator.clone();
     BackgroundOps::new(
         move || all_sync.disc_sync(),
@@ -173,23 +157,23 @@ fn create_background_ops(
 }
 
 impl PersyImpl {
-    pub fn create(path: &Path) -> PRes<()> {
+    pub fn create(path: &Path) -> Result<(), CreateError> {
         let f = OpenOptions::new().write(true).read(true).create_new(true).open(path)?;
         PersyImpl::create_from_file(f)
     }
 
-    pub fn create_from_file(f: File) -> PRes<()> {
+    pub fn create_from_file(f: File) -> Result<(), CreateError> {
         f.try_lock_exclusive()?;
         PersyImpl::init_file(f)?;
         Ok(())
     }
 
-    fn init_file(fl: File) -> PRes<()> {
+    fn init_file(fl: File) -> PERes<()> {
         PersyImpl::init(Box::new(DiscRef::new(fl)?))?;
         Ok(())
     }
 
-    fn init(device: Box<dyn Device>) -> PRes<Box<dyn Device>> {
+    fn init(device: Box<dyn Device>) -> PERes<Box<dyn Device>> {
         // root_page is every time 0
         let root_page = device.create_page_raw(DEFAULT_PAGE_EXP)?;
         let (allocator_page, allocator) = Allocator::init(device, &Config::new())?;
@@ -204,6 +188,7 @@ impl PersyImpl {
             // Start of the Log data, if shutdown well this will be every time 0
             root.write_u64(journal_page);
             root.write_u64(allocator_page);
+            root.write_all(PERSY_TAG_BYTES);
             allocator.flush_page(root)?;
             // TODO: check this never go over the first page
         }
@@ -211,7 +196,7 @@ impl PersyImpl {
         Ok(allocator.release())
     }
 
-    fn new(disc: Box<dyn Device>, config: Config) -> PRes<PersyImpl> {
+    fn new(disc: Box<dyn Device>, config: Config) -> Result<PersyImpl, OpenError> {
         let address_page;
         let journal_page;
         let allocator_page;
@@ -221,6 +206,12 @@ impl PersyImpl {
             address_page = pg.read_u64();
             journal_page = pg.read_u64();
             allocator_page = pg.read_u64();
+            let mut buff = [0u8; 4];
+            pg.read_exact(&mut buff);
+            // Checking alternatively the for bytes to be 0 for backward compatibility
+            if buff != *PERSY_TAG_BYTES && buff != [0u8; 4] {
+                return Err(OpenError::NotPersyFile);
+            }
         }
         let config = Arc::new(config);
         let allocator = Arc::new(Allocator::new(disc, &config, allocator_page)?);
@@ -242,14 +233,14 @@ impl PersyImpl {
         })
     }
 
-    fn recover(&self) -> PRes<RecoverImpl> {
+    fn recover(&self) -> PERes<RecoverImpl> {
         let mut commit_order = Vec::new();
         let mut transactions = HashMap::new();
         let journal = &self.journal;
         journal.recover(|record, id| {
             let tx = transactions
                 .entry(id.clone())
-                .or_insert_with(|| (RecoverStatus::Started, Transaction::recover(id.clone()), None));
+                .or_insert_with(|| (RecoverStatus::Started, TransactionImpl::recover(id.clone()), None));
             tx.0 = match record.recover(&mut tx.1) {
                 Err(_) => RecoverStatus::Rollback,
                 Ok(_) if tx.0 == RecoverStatus::Rollback => RecoverStatus::Rollback,
@@ -279,15 +270,18 @@ impl PersyImpl {
             order: commit_order,
         })
     }
-    pub fn final_recover(&self, mut recover: RecoverImpl) -> PRes<()> {
+
+    pub fn final_recover(&self, mut recover: RecoverImpl) -> PERes<()> {
         let allocator = &self.allocator;
         for id in recover.order {
             if let Some((status, mut tx, choosed)) = recover.transactions.remove(&id) {
                 if status == RecoverStatus::PrepareCommit {
                     if choosed == Some(CommitStatus::Commit) || choosed.is_none() {
-                        let prepared = tx.recover_prepare(self)?;
-                        tx.recover_commit(self, prepared)?;
-                        self.journal.finished_to_clean(&[id])?;
+                        let prepared_result = tx.recover_prepare(self);
+                        if let Ok(prepared) = prepared_result {
+                            tx.recover_commit(self, prepared)?;
+                            self.journal.finished_to_clean(&[id])?;
+                        }
                     } else {
                         tx.recover_rollback(self)?;
                     }
@@ -308,19 +302,19 @@ impl PersyImpl {
         Ok(())
     }
 
-    pub fn open_recover(f: File, config: Config) -> PRes<(PersyImpl, RecoverImpl)> {
+    pub fn open_recover(f: File, config: Config) -> Result<(PersyImpl, RecoverImpl), OpenError> {
         f.try_lock_exclusive()?;
         let persy = PersyImpl::new(Box::new(DiscRef::new(f)?), config)?;
         let rec = persy.recover()?;
         Ok((persy, rec))
     }
 
-    pub fn memory(config: Config) -> PRes<PersyImpl> {
+    pub fn memory(config: Config) -> Result<PersyImpl, OpenError> {
         let device = PersyImpl::init(Box::new(MemRef::new()?))?;
         PersyImpl::new(device, config)
     }
 
-    pub fn begin_with(&self, mut config: TransactionConfig) -> PRes<Transaction> {
+    pub fn begin_with(&self, mut config: TransactionConfig) -> PERes<TransactionImpl> {
         let journal = &self.journal;
         let strategy = if let Some(st) = config.tx_strategy {
             st
@@ -338,18 +332,18 @@ impl PersyImpl {
             SyncMode::Sync
         };
 
-        Ok(Transaction::new(journal, &strategy, sync_mode, meta_id)?)
+        TransactionImpl::new(journal, &strategy, sync_mode, meta_id)
     }
 
-    pub fn create_segment(&self, tx: &mut Transaction, segment: &str) -> PRes<SegmentId> {
+    pub fn create_segment(&self, tx: &mut TransactionImpl, segment: &str) -> Result<SegmentId, CreateSegmentError> {
         match tx.exists_segment(segment) {
-            DROPPED => {}
-            CREATED(_) => {
-                return Err(PersyError::SegmentAlreadyExists);
+            TxSegCheck::Dropped => {}
+            TxSegCheck::Created(_) => {
+                return Err(CreateSegmentError::SegmentAlreadyExists);
             }
-            NONE => {
-                if self.address.exists_segment(&segment)? {
-                    return Err(PersyError::SegmentAlreadyExists);
+            TxSegCheck::None => {
+                if self.address.exists_segment(segment)? {
+                    return Err(CreateSegmentError::SegmentAlreadyExists);
                 }
             }
         }
@@ -358,67 +352,79 @@ impl PersyImpl {
         Ok(segment_id)
     }
 
-    pub fn drop_segment(&self, tx: &mut Transaction, segment: &str) -> PRes<()> {
+    pub fn drop_segment(&self, tx: &mut TransactionImpl, segment: &str) -> Result<(), DropSegmentError> {
         let (_, segment_id) = self.check_segment_tx(tx, segment)?;
         tx.add_drop_segment(&self.journal, segment, segment_id)?;
         Ok(())
     }
 
-    pub fn exists_segment(&self, segment: &str) -> PRes<bool> {
+    pub fn exists_segment(&self, segment: &str) -> PERes<bool> {
         self.address.exists_segment(segment)
     }
 
-    pub fn exists_segment_tx(&self, tx: &Transaction, segment: &str) -> PRes<bool> {
+    pub fn exists_segment_tx(&self, tx: &TransactionImpl, segment: &str) -> PERes<bool> {
         match tx.exists_segment(segment) {
-            DROPPED => Ok(false),
-            CREATED(_) => Ok(true),
-            NONE => self.address.exists_segment(segment),
+            TxSegCheck::Dropped => Ok(false),
+            TxSegCheck::Created(_) => Ok(true),
+            TxSegCheck::None => self.address.exists_segment(segment),
         }
     }
-    pub fn exists_index(&self, index: &str) -> PRes<bool> {
+    pub fn exists_index(&self, index: &str) -> PERes<bool> {
         self.exists_segment(&format!("{}{}", INDEX_META_PREFIX, index))
     }
 
-    pub fn exists_index_tx(&self, tx: &Transaction, index: &str) -> PRes<bool> {
+    pub fn exists_index_tx(&self, tx: &TransactionImpl, index: &str) -> PERes<bool> {
         self.exists_segment_tx(tx, &format!("{}{}", INDEX_META_PREFIX, index))
     }
 
-    pub fn solve_segment_id(&self, segment: impl ToSegmentId) -> PRes<SegmentId> {
+    pub fn solve_segment_id(&self, segment: impl ToSegmentId) -> Result<SegmentId, SegmentError> {
         segment.to_segment_id(&self.address)
     }
 
-    pub fn solve_segment_id_tx(&self, tx: &Transaction, segment: impl ToSegmentId) -> PRes<SegmentId> {
+    pub fn solve_segment_id_tx(
+        &self,
+        tx: &TransactionImpl,
+        segment: impl ToSegmentId,
+    ) -> Result<SegmentId, SegmentError> {
         let (sid, _) = segment.to_segment_id_tx(self, tx)?;
         Ok(sid)
     }
 
-    pub fn solve_segment_id_snapshot(&self, snapshot: SnapshotId, segment: impl ToSegmentId) -> PRes<SegmentId> {
+    pub fn solve_segment_id_snapshot(
+        &self,
+        snapshot: SnapshotId,
+        segment: impl ToSegmentId,
+    ) -> Result<SegmentId, SegmentError> {
         segment.to_segment_id_snapshot(&self.snapshots, snapshot)
     }
 
-    pub fn solve_index_id(&self, index: impl ToIndexId) -> PRes<IndexId> {
+    pub fn solve_index_id(&self, index: impl ToIndexId) -> Result<IndexId, IndexError> {
         index.to_index_id(&self.address)
     }
 
-    pub fn solve_index_id_tx(&self, tx: &Transaction, index: impl ToIndexId) -> PRes<(IndexId, bool)> {
+    pub fn solve_index_id_tx(
+        &self,
+        tx: &TransactionImpl,
+        index: impl ToIndexId,
+    ) -> Result<(IndexId, bool), IndexError> {
         index.to_index_id_tx(self, tx)
     }
 
-    pub fn solve_index_id_snapshot(&self, snapshot: SnapshotId, index: impl ToIndexId) -> PRes<IndexId> {
+    pub fn solve_index_id_snapshot(&self, snapshot: SnapshotId, index: impl ToIndexId) -> Result<IndexId, IndexError> {
         index.to_index_id_snapshot(&self.snapshots, snapshot)
     }
 
     /// check if a segment exist persistent or in tx.
     ///
     /// @return true if the segment was created in tx.
-    pub fn check_segment_tx(&self, tx: &Transaction, segment: &str) -> PRes<(bool, SegmentId)> {
+    pub fn check_segment_tx(&self, tx: &TransactionImpl, segment: &str) -> Result<(bool, SegmentId), SegmentError> {
         match tx.exists_segment(segment) {
-            DROPPED => Err(PersyError::SegmentNotFound),
-            CREATED(segment_id) => Ok((true, segment_id)),
-            NONE => self
+            TxSegCheck::Dropped => Err(SegmentError::SegmentNotFound),
+            TxSegCheck::Created(segment_id) => Ok((true, segment_id)),
+            TxSegCheck::None => self
                 .address
                 .segment_id(segment)?
-                .map_or(Err(PersyError::SegmentNotFound), |id| Ok((false, id))),
+                .map_or(Err(SegmentError::SegmentNotFound), |id| Ok((false, id))),
         }
     }
 
@@ -436,16 +442,21 @@ impl PersyImpl {
         (len, id)
     }
 
-    pub fn insert_record(&self, tx: &mut Transaction, segment: impl ToSegmentId, rec: &[u8]) -> PRes<RecRef> {
+    pub fn insert_record(
+        &self,
+        tx: &mut TransactionImpl,
+        segment: impl ToSegmentId,
+        rec: &[u8],
+    ) -> Result<RecRef, SegmentError> {
         let (segment_id, in_tx) = segment.to_segment_id_tx(self, tx)?;
         let len = rec.len() as u64;
         let allocator = &self.allocator;
         let address = &self.address;
         let (rec_ref, maybe_new_page) = if in_tx {
-            address.allocate_temp(segment_id)
+            address.allocate_temp(segment_id)?
         } else {
-            address.allocate(segment_id)
-        }?;
+            address.allocate(segment_id)?
+        };
         let metadata = PersyImpl::write_record_metadata(len, &rec_ref);
         let allocation_exp = exp_from_content_size(len + metadata.len() as u64);
         let mut pg = allocator.allocate(allocation_exp)?;
@@ -460,11 +471,11 @@ impl PersyImpl {
         Ok(rec_ref)
     }
 
-    fn read_snapshot(&self) -> PRes<SnapshotId> {
+    fn read_snapshot(&self) -> PERes<SnapshotId> {
         self.snapshots.current_snapshot()
     }
 
-    pub fn snapshot(&self) -> PRes<SnapshotId> {
+    pub fn snapshot(&self) -> PERes<SnapshotId> {
         let snapshot_id = self.snapshots.read_snapshot()?;
         let segs = self
             .address
@@ -476,31 +487,31 @@ impl PersyImpl {
         Ok(snapshot_id)
     }
 
-    pub fn release_snapshot(&self, snapshot_id: SnapshotId) -> PRes<()> {
+    pub fn release_snapshot(&self, snapshot_id: SnapshotId) -> PERes<()> {
         release_snapshot(snapshot_id, &self.snapshots, &self.allocator, &self.journal)
     }
 
     fn read_ref_segment(
         &self,
-        tx: &Transaction,
+        tx: &TransactionImpl,
         segment_id: SegmentId,
         rec_ref: &RecRef,
-    ) -> PRes<Option<(u64, u16, SegmentId)>> {
+    ) -> Result<Option<(u64, u16, SegmentId)>, ReadError> {
         Ok(match tx.read(rec_ref) {
-            TxRead::RECORD(rec) => Some((rec.0, rec.1, segment_id)),
-            TxRead::DELETED => None,
-            TxRead::NONE => self
+            TxRead::Record(rec) => Some((rec.0, rec.1, segment_id)),
+            TxRead::Deleted => None,
+            TxRead::None => self
                 .address
                 .read(rec_ref, segment_id)?
                 .map(|(pos, version)| (pos, version, segment_id)),
         })
     }
 
-    fn read_page(&self, match_id: &RecRef, page: u64) -> PRes<Option<Vec<u8>>> {
+    fn read_page(&self, match_id: &RecRef, page: u64) -> PERes<Option<Vec<u8>>> {
         self.read_page_fn(match_id, page, |x| Vec::from(x))
     }
 
-    fn read_page_fn<T>(&self, match_id: &RecRef, page: u64, f: fn(&[u8]) -> T) -> PRes<Option<T>> {
+    fn read_page_fn<T, F: Fn(&[u8]) -> T>(&self, match_id: &RecRef, page: u64, f: F) -> PERes<Option<T>> {
         if let Some(mut pg) = self.allocator.load_page_not_free(page)? {
             let (len, id) = PersyImpl::read_record_metadata(&mut pg);
             if id.page == match_id.page && id.pos == match_id.pos {
@@ -515,16 +526,24 @@ impl PersyImpl {
 
     pub fn read_tx_internal_fn<T>(
         &self,
-        tx: &Transaction,
+        tx: &TransactionImpl,
         segment_id: SegmentId,
         id: &RecRef,
         f: fn(&[u8]) -> T,
-    ) -> PRes<Option<(T, u16)>> {
+    ) -> Result<Option<(T, u16)>, ReadError> {
+        let mut retry_count = 0;
         loop {
             if let Some((page, version, _)) = self.read_ref_segment(tx, segment_id, id)? {
                 if let Some(record) = self.read_page_fn(id, page, f)? {
                     break Ok(Some((record, version)));
                 }
+                debug_assert!(
+                    retry_count < 100,
+                    "more than 100 retry probably means an issue id:{} page:{}",
+                    id,
+                    page
+                );
+                retry_count += 1;
             } else {
                 break Ok(None);
             }
@@ -533,14 +552,19 @@ impl PersyImpl {
 
     pub fn read_tx_internal(
         &self,
-        tx: &Transaction,
+        tx: &TransactionImpl,
         segment_id: SegmentId,
         id: &RecRef,
-    ) -> PRes<Option<(Vec<u8>, u16)>> {
+    ) -> Result<Option<(Vec<u8>, u16)>, ReadError> {
         self.read_tx_internal_fn(tx, segment_id, id, |x| Vec::from(x))
     }
 
-    pub fn read_tx(&self, tx: &mut Transaction, segment: SegmentId, id: &RecRef) -> PRes<Option<Vec<u8>>> {
+    pub fn read_tx(
+        &self,
+        tx: &mut TransactionImpl,
+        segment: SegmentId,
+        id: &RecRef,
+    ) -> Result<Option<Vec<u8>>, ReadError> {
         if let Some((rec, version)) = self.read_tx_internal(tx, segment, id)? {
             tx.add_read(&self.journal, segment, id, version)?;
             Ok(Some(rec))
@@ -551,33 +575,44 @@ impl PersyImpl {
 
     pub fn lock_record(
         &self,
-        tx: &mut Transaction,
-        segment: impl ToSegmentId,
+        tx: &mut TransactionImpl,
+        segment_id: SegmentId,
         id: &RecRef,
         version: u16,
-    ) -> PRes<bool> {
-        let segment_id = segment.to_segment_id(&self.address)?;
+    ) -> Result<bool, TimeoutError> {
         tx.lock_record(&self.address, segment_id, id, version)
     }
 
-    pub fn unlock_record(&self, tx: &mut Transaction, segment: impl ToSegmentId, id: &RecRef) -> PRes<()> {
-        let segment_id = segment.to_segment_id(&self.address)?;
+    pub fn unlock_record(&self, tx: &mut TransactionImpl, segment_id: SegmentId, id: &RecRef) -> PERes<()> {
         tx.unlock_record(&self.address, segment_id, id)
     }
 
-    pub fn read(&self, segment: SegmentId, rec_ref: &RecRef) -> PRes<Option<Vec<u8>>> {
+    pub fn read(&self, segment: SegmentId, rec_ref: &RecRef) -> Result<Option<Vec<u8>>, ReadError> {
+        let mut retry_count = 0;
         loop {
             if let Some((page, _)) = self.address.read(rec_ref, segment)? {
                 if let Some(record) = self.read_page(rec_ref, page)? {
                     break Ok(Some(record));
                 }
+                debug_assert!(
+                    retry_count < 100,
+                    "more than 100 retry probably means an issue id:{} page:{}",
+                    rec_ref,
+                    page
+                );
+                retry_count += 1;
             } else {
                 break Ok(None);
             }
         }
     }
 
-    pub fn read_snap(&self, segment: SegmentId, rec_ref: &RecRef, snapshot: SnapshotId) -> PRes<Option<Vec<u8>>> {
+    pub fn read_snap(
+        &self,
+        segment: SegmentId,
+        rec_ref: &RecRef,
+        snapshot: SnapshotId,
+    ) -> Result<Option<Vec<u8>>, ReadError> {
         self.read_snap_fn(segment, rec_ref, snapshot, |x| Vec::from(x))
     }
 
@@ -587,36 +622,40 @@ impl PersyImpl {
         rec_ref: &RecRef,
         snapshot: SnapshotId,
         f: fn(&[u8]) -> T,
-    ) -> PRes<Option<T>> {
+    ) -> Result<Option<T>, ReadError> {
         let segment_id = segment;
-        loop {
-            if let Some(rec_vers) = self.snapshots.read(snapshot, rec_ref)? {
-                match rec_vers.case {
-                    EntryCase::Change(change) => {
-                        if let Some(record) = self.read_page_fn(rec_ref, change.pos, f)? {
-                            break Ok(Some(record));
-                        }
-                    }
-                    EntryCase::Insert => {
-                        break Ok(None);
+        if let Some(rec_vers) = self.snapshots.read(snapshot, rec_ref)? {
+            match rec_vers.case {
+                EntryCase::Change(change) => {
+                    if let Some(record) = self.read_page_fn(rec_ref, change.pos, f)? {
+                        Ok(Some(record))
+                    } else {
+                        panic!("if page do not match the content the snapshot is broken");
                     }
                 }
-            } else if let Some((page, _)) = self.address.read(rec_ref, segment_id)? {
-                if let Some(record) = self.read_page_fn(rec_ref, page, f)? {
-                    break Ok(Some(record));
-                }
-            } else {
-                break Ok(None);
+                EntryCase::Insert => Ok(None),
             }
+        } else if let Some((page, _)) = self.address.read(rec_ref, segment_id)? {
+            if let Some(record) = self.read_page_fn(rec_ref, page, f)? {
+                Ok(Some(record))
+            } else {
+                panic!("if page do not match the content the snapshot is broken");
+            }
+        } else {
+            Ok(None)
         }
     }
 
-    pub fn scan(&self, segment: SegmentId) -> PRes<SegmentRawIter> {
+    pub fn scan(&self, segment: SegmentId) -> Result<SegmentRawIter, SegmentError> {
         let read_snapshot = self.read_snapshot()?;
         Ok(SegmentRawIter::new(segment, self.address.scan(segment)?, read_snapshot))
     }
 
-    pub fn scan_snapshot_index(&self, segment_id: SegmentId, snapshot: SnapshotId) -> PRes<SegmentSnapshotRawIter> {
+    pub fn scan_snapshot_index(
+        &self,
+        segment_id: SegmentId,
+        snapshot: SnapshotId,
+    ) -> Result<SegmentSnapshotRawIter, SegmentError> {
         let res = if let Some(r) = self.snapshots.scan(snapshot, segment_id)? {
             r
         } else {
@@ -625,12 +664,16 @@ impl PersyImpl {
         Ok(SegmentSnapshotRawIter::new(segment_id, res, snapshot))
     }
 
-    pub fn scan_snapshot(&self, segment_id: SegmentId, snapshot: SnapshotId) -> PRes<SegmentSnapshotRawIter> {
+    pub fn scan_snapshot(&self, segment_id: SegmentId, snapshot: SnapshotId) -> PERes<SegmentSnapshotRawIter> {
         let res = self.snapshots.scan(snapshot, segment_id)?.unwrap();
         Ok(SegmentSnapshotRawIter::new(segment_id, res, snapshot))
     }
 
-    pub fn scan_tx<'a>(&'a self, tx: &'a Transaction, segment_id: SegmentId) -> PRes<TxSegmentRawIter> {
+    pub fn scan_tx<'a>(
+        &'a self,
+        tx: &'a TransactionImpl,
+        segment_id: SegmentId,
+    ) -> Result<TxSegmentRawIter, SegmentError> {
         let read_snapshot = self.read_snapshot()?;
         Ok(TxSegmentRawIter::new(
             tx,
@@ -640,37 +683,43 @@ impl PersyImpl {
         ))
     }
 
-    pub fn update(&self, tx: &mut Transaction, segment: SegmentId, rec_ref: &RecRef, rec: &[u8]) -> PRes<()> {
+    pub fn update(
+        &self,
+        tx: &mut TransactionImpl,
+        segment: SegmentId,
+        rec_ref: &RecRef,
+        rec: &[u8],
+    ) -> Result<(), UpdateError> {
         if let Some((_, version, segment)) = self.read_ref_segment(tx, segment, rec_ref)? {
             let allocator = &self.allocator;
             let journal = &self.journal;
             let len = rec.len();
-            let metadata = PersyImpl::write_record_metadata(len as u64, &rec_ref);
+            let metadata = PersyImpl::write_record_metadata(len as u64, rec_ref);
             let allocation_exp = exp_from_content_size((len + metadata.len()) as u64);
             let mut pg = allocator.allocate(allocation_exp)?;
             let page = pg.get_index();
-            tx.add_update(journal, segment, &rec_ref, page, version)?;
+            tx.add_update(journal, segment, rec_ref, page, version)?;
             pg.write_all(&metadata);
             pg.write_all(rec);
-            allocator.flush_page(pg)
+            Ok(allocator.flush_page(pg)?)
         } else {
-            Err(PersyError::RecordNotFound(PersyId(rec_ref.clone())))
+            Err(UpdateError::RecordNotFound(PersyId(rec_ref.clone())))
         }
     }
 
-    pub fn delete(&self, tx: &mut Transaction, segment: SegmentId, rec_ref: &RecRef) -> PRes<()> {
+    pub fn delete(&self, tx: &mut TransactionImpl, segment: SegmentId, rec_ref: &RecRef) -> Result<(), DeleteError> {
         if let Some((_, version, seg)) = self.read_ref_segment(tx, segment, rec_ref)? {
-            tx.add_delete(&self.journal, seg, &rec_ref, version)
+            Ok(tx.add_delete(&self.journal, seg, rec_ref, version)?)
         } else {
-            Err(PersyError::RecordNotFound(PersyId(rec_ref.clone())))
+            Err(DeleteError::RecordNotFound(PersyId(rec_ref.clone())))
         }
     }
 
-    pub fn rollback(&self, mut tx: Transaction) -> PRes<()> {
+    pub fn rollback(&self, mut tx: TransactionImpl) -> PERes<()> {
         tx.rollback(self)
     }
 
-    pub fn prepare(&self, tx: Transaction) -> PRes<TxFinalize> {
+    pub fn prepare(&self, tx: TransactionImpl) -> Result<TxFinalize, PrepareError> {
         let (tx, prepared) = tx.prepare(self)?;
 
         Ok(TxFinalize {
@@ -679,19 +728,24 @@ impl PersyImpl {
         })
     }
 
-    pub fn rollback_prepared(&self, finalizer: &mut TxFinalize) -> PRes<()> {
+    pub fn rollback_prepared(&self, finalizer: &mut TxFinalize) -> PERes<()> {
         let prepared = finalizer.prepared.clone();
         let tx = &mut finalizer.transaction;
         tx.rollback_prepared(self, prepared)
     }
 
-    pub fn commit(&self, finalizer: &mut TxFinalize) -> PRes<()> {
+    pub fn commit(&self, finalizer: &mut TxFinalize) -> PERes<()> {
         let prepared = finalizer.prepared.clone();
         let tx = &mut finalizer.transaction;
         tx.commit(self, prepared)
     }
 
-    pub fn create_index<K, V>(&self, tx: &mut Transaction, index_name: &str, value_mode: ValueMode) -> PRes<()>
+    pub fn create_index<K, V>(
+        &self,
+        tx: &mut TransactionImpl,
+        index_name: &str,
+        value_mode: ValueMode,
+    ) -> Result<(), CreateIndexError>
     where
         K: IndexType,
         V: IndexType,
@@ -699,11 +753,11 @@ impl PersyImpl {
         Indexes::create_index::<K, V>(self, tx, index_name, 32, 128, value_mode)
     }
 
-    pub fn drop_index(&self, tx: &mut Transaction, index_name: &str) -> PRes<()> {
+    pub fn drop_index(&self, tx: &mut TransactionImpl, index_name: &str) -> Result<(), DropIndexError> {
         Indexes::drop_index(self, tx, index_name)
     }
 
-    pub fn put<K, V>(&self, tx: &mut Transaction, index_id: IndexId, k: K, v: V) -> PRes<()>
+    pub fn put<K, V>(&self, tx: &mut TransactionImpl, index_id: IndexId, k: K, v: V) -> Result<(), IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
@@ -713,7 +767,13 @@ impl PersyImpl {
         Ok(())
     }
 
-    pub fn remove<K, V>(&self, tx: &mut Transaction, index_id: IndexId, k: K, v: Option<V>) -> PRes<()>
+    pub fn remove<K, V>(
+        &self,
+        tx: &mut TransactionImpl,
+        index_id: IndexId,
+        k: K,
+        v: Option<V>,
+    ) -> Result<(), IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
@@ -723,7 +783,12 @@ impl PersyImpl {
         Ok(())
     }
 
-    pub fn get_tx<K, V>(&self, tx: &mut Transaction, index_id: IndexId, k: &K) -> PRes<Option<Value<V>>>
+    pub fn get_tx<K, V>(
+        &self,
+        tx: &mut TransactionImpl,
+        index_id: IndexId,
+        k: &K,
+    ) -> Result<Option<Value<V>>, IndexChangeError>
     where
         K: IndexType,
         V: IndexType,
@@ -737,7 +802,7 @@ impl PersyImpl {
         tx.apply_changes::<K, V>(vm, index_id, k, result)
     }
 
-    pub fn get<K, V>(&self, index_id: IndexId, k: &K) -> PRes<Option<Value<V>>>
+    pub fn get<K, V>(&self, index_id: IndexId, k: &K) -> Result<Option<Value<V>>, IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
@@ -748,12 +813,17 @@ impl PersyImpl {
         r
     }
 
-    pub fn get_snapshot<K, V>(&self, index_id: IndexId, snapshot: SnapshotId, k: &K) -> PRes<Option<Value<V>>>
+    pub fn get_snapshot<K, V>(
+        &self,
+        index_id: IndexId,
+        snapshot: SnapshotId,
+        k: &K,
+    ) -> Result<Option<Value<V>>, IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
     {
-        Indexes::get_index_keeper::<K, V>(self, snapshot, &index_id)?.get(k)
+        Ok(Indexes::get_index_keeper::<K, V>(self, snapshot, &index_id)?.get(k)?)
     }
 
     pub fn index_next<K, V>(
@@ -761,12 +831,12 @@ impl PersyImpl {
         index_id: &IndexId,
         read_snapshot: SnapshotId,
         next: Bound<&K>,
-    ) -> PRes<PageIter<K, V>>
+    ) -> Result<PageIter<K, V>, IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
     {
-        Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.iter_from(next)
+        Ok(Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.iter_from(next)?)
     }
 
     pub fn index_back<K, V>(
@@ -774,15 +844,15 @@ impl PersyImpl {
         index_id: &IndexId,
         read_snapshot: SnapshotId,
         next: Bound<&K>,
-    ) -> PRes<PageIterBack<K, V>>
+    ) -> Result<PageIterBack<K, V>, IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
     {
-        Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.back_iter_from(next)
+        Ok(Indexes::get_index_keeper::<K, V>(self, read_snapshot, index_id)?.back_iter_from(next)?)
     }
 
-    pub fn range<K, V, R>(&self, index_id: IndexId, range: R) -> PRes<(ValueMode, IndexRawIter<K, V>)>
+    pub fn range<K, V, R>(&self, index_id: IndexId, range: R) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
@@ -798,7 +868,7 @@ impl PersyImpl {
         snapshot: SnapshotId,
         range: R,
         release_snapshot: bool,
-    ) -> PRes<(ValueMode, IndexRawIter<K, V>)>
+    ) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
     where
         K: IndexType,
         V: IndexType,
@@ -813,7 +883,28 @@ impl PersyImpl {
         ))
     }
 
-    pub fn segment_name_tx(&self, tx: &Transaction, id: SegmentId) -> PRes<Option<(String, bool)>> {
+    pub fn range_tx<K, V, R>(
+        &self,
+        tx: &mut TransactionImpl,
+        index_id: IndexId,
+        range: R,
+    ) -> Result<(ValueMode, IndexRawIter<K, V>), IndexOpsError>
+    where
+        K: IndexType,
+        V: IndexType,
+        R: RangeBounds<K>,
+    {
+        let mut ik = Indexes::get_index_keeper_tx::<K, V>(self, tx, &index_id)?;
+        let after = ik.iter_from(range.start_bound())?;
+        let before = ik.back_iter_from(range.end_bound())?;
+        let snapshot = self.snapshots.read_snapshot()?;
+        Ok((
+            IndexKeeper::<K, V>::value_mode(&ik),
+            IndexRawIter::new(index_id, snapshot, after, before, true),
+        ))
+    }
+
+    pub fn segment_name_tx(&self, tx: &TransactionImpl, id: SegmentId) -> PERes<Option<(String, bool)>> {
         if tx.segment_created_in_tx(id) {
             Ok(tx.segment_name_by_id(id).map(|x| (x, true)))
         } else {
@@ -821,7 +912,7 @@ impl PersyImpl {
         }
     }
 
-    pub fn list_segments(&self) -> PRes<Vec<(String, SegmentId)>> {
+    pub fn list_segments(&self) -> PERes<Vec<(String, SegmentId)>> {
         Ok(self
             .address
             .list()?
@@ -830,7 +921,7 @@ impl PersyImpl {
             .collect())
     }
 
-    pub fn list_segments_snapshot(&self, snapshot_id: SnapshotId) -> PRes<Vec<(String, SegmentId)>> {
+    pub fn list_segments_snapshot(&self, snapshot_id: SnapshotId) -> PERes<Vec<(String, SegmentId)>> {
         let list = self.snapshots.list(snapshot_id)?;
         Ok(list
             .into_iter()
@@ -838,26 +929,26 @@ impl PersyImpl {
             .collect())
     }
 
-    pub fn list_indexes(&self) -> PRes<Vec<(String, IndexInfo)>> {
+    pub fn list_indexes(&self) -> PERes<Vec<(String, IndexInfo)>> {
         let snapshot = self.snapshot()?;
         let res = self.list_indexes_snapshot(snapshot);
         release_snapshot(snapshot, &self.snapshots, &self.allocator, &self.journal)?;
         res
     }
 
-    pub fn list_indexes_snapshot(&self, snapshot: SnapshotId) -> PRes<Vec<(String, IndexInfo)>> {
+    pub fn list_indexes_snapshot(&self, snapshot: SnapshotId) -> PERes<Vec<(String, IndexInfo)>> {
         let list = self.snapshots.list(snapshot)?;
-        list.into_iter()
+        Ok(list
+            .into_iter()
             .filter(|(name, _)| name.starts_with(INDEX_META_PREFIX))
-            .map(|(mut name, _id)| -> PRes<(String, IndexInfo)> {
+            .filter_map(|(mut name, _id)| -> Option<(String, IndexInfo)> {
                 name.drain(..INDEX_META_PREFIX.len());
-                let info = self.index_info(snapshot, &name)?;
-                Ok((name, info))
+                self.index_info(snapshot, &name).ok().map(|info| (name, info))
             })
-            .collect()
+            .collect())
     }
 
-    pub fn list_segments_tx(&self, tx: &Transaction) -> PRes<Vec<(String, SegmentId)>> {
+    pub fn list_segments_tx(&self, tx: &TransactionImpl) -> PERes<Vec<(String, SegmentId)>> {
         Ok(tx
             .filter_list(&self.address.list()?)
             .filter(|(name, _)| !name.starts_with(INDEX_META_PREFIX) && !name.starts_with(INDEX_DATA_PREFIX))
@@ -865,19 +956,19 @@ impl PersyImpl {
             .collect())
     }
 
-    pub fn list_indexes_tx(&self, tx: &Transaction) -> PRes<Vec<(String, IndexInfo)>> {
-        tx.filter_list(&self.address.list()?)
+    pub fn list_indexes_tx(&self, tx: &TransactionImpl) -> PERes<Vec<(String, IndexInfo)>> {
+        Ok(tx
+            .filter_list(&self.address.list()?)
             .filter(|(name, _)| name.starts_with(INDEX_META_PREFIX))
             .map(|(name, id)| (name.to_string(), id))
-            .map(|(mut name, _id)| -> PRes<(String, IndexInfo)> {
+            .filter_map(|(mut name, _id)| -> Option<(String, IndexInfo)> {
                 name.drain(..INDEX_META_PREFIX.len());
-                let info = self.index_info_tx(tx, &name)?;
-                Ok((name, info))
+                self.index_info_tx(tx, &name).ok().map(|info| (name, info))
             })
-            .collect()
+            .collect())
     }
 
-    fn index_info(&self, snapshot: SnapshotId, name: &str) -> PRes<IndexInfo> {
+    fn index_info(&self, snapshot: SnapshotId, name: &str) -> Result<IndexInfo, IndexError> {
         let id = self.solve_index_id_snapshot(snapshot, name)?;
         let index = Indexes::get_index(self, snapshot, &id)?;
         Ok(IndexInfo {
@@ -887,7 +978,7 @@ impl PersyImpl {
             value_type: IndexTypeId::from(index.value_type),
         })
     }
-    fn index_info_tx(&self, tx: &Transaction, name: &str) -> PRes<IndexInfo> {
+    fn index_info_tx(&self, tx: &TransactionImpl, name: &str) -> Result<IndexInfo, IndexError> {
         let id = self.solve_index_id_tx(tx, name)?.0;
         let (index, _version) = Indexes::get_index_tx(self, tx, &id)?;
         Ok(IndexInfo {
@@ -919,7 +1010,7 @@ impl PersyImpl {
     }
 
     #[cfg(feature = "background_ops")]
-    pub(crate) fn transaction_sync(&self, sync_mode: &SyncMode, snapshot_id: SnapshotId) -> PRes<()> {
+    pub(crate) fn transaction_sync(&self, sync_mode: &SyncMode, snapshot_id: SnapshotId) -> PERes<()> {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
@@ -933,7 +1024,7 @@ impl PersyImpl {
     }
 
     #[cfg(not(feature = "background_ops"))]
-    pub(crate) fn transaction_sync(&self, sync_mode: &SyncMode, _snapshot_id: SnapshotId) -> PRes<()> {
+    pub(crate) fn transaction_sync(&self, sync_mode: &SyncMode, _snapshot_id: SnapshotId) -> PERes<()> {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
